@@ -2,12 +2,25 @@
 # MKDoc::Site::Deploy::DB
 # -----------------------------------------------------------------------------
 #
-#       Author : Jean-Michel Hiver <jhiver@mkdoc.com>
+#       Author : Jean-Michel Hiver
 #                Steve Purkis <spurkis@mkdoc.com>
 #    Copyright : (c) MKDoc Holdings Ltd, 2001
 #
-#      Unauthorized modification, use, reuse, distribution or redistribution
-#      of this module is stricly forbidden
+# This file is part of MKDoc. 
+# 
+# MKDoc is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+# 
+# MKDoc is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with MKDoc; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 #    Description:
 #
@@ -36,6 +49,8 @@ use warnings;
 use File::Spec;
 use File::Path;
 
+use Crypt::PassGen;
+
 use MKDoc::Site::Deploy::DB::Driver;
 use MKDoc::Site::Deploy::DB::Schema;
 use Carp;
@@ -63,7 +78,7 @@ sub new
 ##
 
 ## Version (from CVS revision string).
-our $VERSION = (split(/ /, '$Revision: 1.1.2.10 $'))[1];
+our $VERSION = (split(/ /, '$Revision: 1.1.2.13 $'))[1];
 
 
 
@@ -448,19 +463,7 @@ sub populate_tables
     my $document_table      = lib::sql::Table->table('Document');
     my $base_document_table = lib::sql::Table->table('Base_Document');
     
-    my $amount = int (rand (2)) + 3;
-    my @rand = qw /a i u e o
-		   ka ki ku ke ko
-		   sa shi su se so
-		   ta tchi tsu te to
-		   na ni nu ne no
-		   ha hi fu he ho
-		   ma mi mu me mo
-		   ya yu yo
-		   la li lu le lo
-		   wa wo/;
-    
-    $::ADMIN_PASSWD = join '', map { $rand[int rand (scalar @rand)] } 1..$amount;
+    ($::ADMIN_PASSWD) = Crypt::PassGen::passgen();
     
     $self->say("\t  +  adding 'admin' user\n");
     $editor_table
