@@ -364,6 +364,63 @@ new lib::sql::Table
    fk     => { Document => { Document_ID => 'ID' } },
   );
 
+## GRP TABLE - holds user groups.  (The table can't be named Group
+## because group is a reserved word in SQL.)
+new lib::sql::Table
+  (
+   name => 'Grp',
+   pk   => [ 'ID' ],
+   ai   => 1,
+   cols => [
+	    { name => 'ID',
+              type => lib::sql::type::Int->new( not_null => 1 ) },
+	    { name => 'Name',
+              type => lib::sql::type::Char->new( size => 255 ) },
+	    { name => 'Description',
+              type => lib::sql::type::Char->new( size => 255 ) },
+	   ]
+  );
+
+## EDITOR GRP TABLE - holds assignments of editors (users) to
+## groups.  When a user is assigned to a group she can access
+## documents assigned to that group.
+new lib::sql::Table
+  (
+   name => 'Editor_Grp',
+   pk   => [ 'Editor_ID', 'Grp_ID' ],
+   cols => [
+	    { name => 'Editor_ID',
+              type => lib::sql::type::Int->new( not_null => 1 ) },
+	    { name => 'Grp_ID',
+              type => lib::sql::type::Int->new( not_null => 1 ) },
+	   ],
+   fk => {
+	  Grp  => { Grp_ID  => 'ID' },
+	  Editor => { Editor_ID => 'ID' },
+	 },
+   index => { ReverseGrpEditorIndex => [ 'Grp_ID', 'Editor_ID' ] },
+  );
+
+## DOCUMENT GRP TABLE - holds assignments of documents to groups.
+## When a document is assigned to one or more groups only members of
+## that group can access the document.
+new lib::sql::Table
+  (
+   name => 'Document_Grp',
+   pk   => [ 'Document_ID', 'Grp_ID' ],
+   cols => [
+	    { name => 'Document_ID',
+              type => lib::sql::type::Int->new( not_null => 1 ) },
+	    { name => 'Grp_ID',
+              type => lib::sql::type::Int->new( not_null => 1 ) },
+	   ],
+   fk => {
+	  Grp    => { Grp_ID    => 'ID' },
+	  Document => { Document_ID => 'ID' },
+	 },
+   index => { ReverseGrpDocumentIndex => [ 'Grp_ID', 'Document_ID' ] },
+  );
+
 1;
 
 __END__
