@@ -32,6 +32,7 @@ use 5.008_000;
 use utf8;
 
 use constant BLOCK => 'edit_block';
+our $QBLOCK = quotemeta (BLOCK) . "_";   
 
 
 our $IMP = {};
@@ -149,9 +150,11 @@ sub _new
 ##
 # $self->links;
 # -------------
-# This subroutine returns all the links of the current document
-# and caches them into $LINKS, these links can then be used by
-# MKDoc::XML::Tagger to linkify the whole document.
+# This subroutine returns all the links of the current document and
+# caches them into $LINKS, these links can then be used by
+# MKDoc::XML::Tagger to linkify the whole document.  The actual calls
+# to MKDoc::XML::Tagger occur in the components which may contain
+# links - flo::editor::Text and flo::editor::HTML, for example.
 #
 # It may look like a horrible hack (and it is), but since the
 # component system needs to be redesigned it does not bother
@@ -176,12 +179,26 @@ sub links
 	# retrieve the links for that document
 	foreach my $link_component (@links_components)
 	{
+            my $url = $link_component->render_url;
+
+            # setup class based on whether this is an internal or external link
+            my $class = "mkdoc-link-" . ( $link_component->is_internal ? 
+                                          "internal" : "external" );
+
+            # setup target to open a new window in needed
+            my @target;
+            if (MKDoc::Config->EXTERNAL_LINKS_OPEN_IN_NEW_WINDOW() and 
+                not $link_component->is_internal) {
+                @target = (target => "_blank");
+            }
+            
 	    push @links, {
-               class => "mkdoc-link",
-		href => $link_component->url,
-		desc => $link_component->description,
-		expr => $link_component->title,
-	    } if ($link_component->url ne '' and $link_component->title ne '');
+                class => $class,
+		href  => $url,
+		desc  => $link_component->description,
+		expr  => $link_component->title,              
+                @target,
+	    } if ($url ne '' and $link_component->title ne '');
 	}
 	
 	# plus we want to hyperlink all the children
@@ -213,6 +230,25 @@ sub links
 sub access
 {
     my $self = shift;
+   
+    # process the --exclude flag
+    # search @_ for 'exclude' followed by something
+    my %exclude = ();
+    for (my $i = 0; $i < @_; $i++)
+    {
+	if (defined $_[$i] and $_[$i] eq 'exclude')
+	{
+	    my ($key, $val) = splice @_, $i, 2;
+            $exclude{$val}  = $val;
+	}
+    }
+
+    scalar %exclude and do {
+        return map {
+            my $name = $_->name();
+            $exclude{$name} ? () : $_;
+        } $self->access (@_);
+    };
     
     # search @_ for 'negate'
     my $component_negate = 'no';
@@ -345,8 +381,7 @@ sub _initialize
     
     $self->_initialize_initialize ($cgi);
     $self->_initialize_delete ($cgi);
-    $self->_initialize_up ($cgi);
-    $self->_initialize_down ($cgi);
+    $self->_initialize_move ($cgi);
     $self->_initialize_add ($cgi);
     $self->_initialize_uri_names ($cgi);
 }
@@ -416,12 +451,10 @@ sub _initialize_initialize
     # This loop sets all the components by scanning the CGI parameters
     foreach my $cgi_param_name ($cgi->param)
     {
-	my $qBLOCK = quotemeta (BLOCK) . "_";
-
 	# if this CGI param is appropriate because it starts by BLOCK
-	if ($cgi_param_name =~ /^$qBLOCK([0-9]+)_([a-zA-Z]+)(?:_.*)?$/)
+	if ($cgi_param_name =~ /^$QBLOCK([0-9]+)_([a-zA-Z]+)(?:_.*)?$/)
 	{
-	    my ($id, $type) = $cgi_param_name =~ /^$qBLOCK([0-9]+)_([a-zA-Z]+)(?:_.*)?$/;
+	    my ($id, $type) = $cgi_param_name =~ /^$QBLOCK([0-9]+)_([a-zA-Z]+)(?:_.*)?$/;
 	    
 	    # if $self->{block}->{$id} is already defined, then this component
 	    # has been instanciated already and we can safely move on the next
@@ -437,7 +470,7 @@ sub _initialize_initialize
 	    require $file;
 	    import  $file;
 	    
-	    my $param_name = $qBLOCK . $id . '_' . $type;
+	    my $param_name = $QBLOCK . $id . '_' . $type;
 	    $self->{block}->{$id} = $class->new ( cgi => $cgi, param_name => $param_name );
 	}
     }
@@ -459,11 +492,9 @@ sub _initialize_delete
     # a component is requested
     foreach my $cgi_param_name ($cgi->param)
     {
-	my $qBLOCK = quotemeta (BLOCK) . "_";
-
-	if ($cgi_param_name =~ /^$qBLOCK([0-9]+)_([a-zA-Z]+)_delete$/)
+	if ($cgi_param_name =~ /^$QBLOCK([0-9]+)_([a-zA-Z]+)_delete$/)
 	{
-	    my ($id, $type) = $cgi_param_name =~ /^$qBLOCK([0-9]+)_([a-zA-Z]+)_delete$/;
+	    my ($id, $type) = $cgi_param_name =~ /^$QBLOCK([0-9]+)_([a-zA-Z]+)_delete$/;
 	    delete $self->{block}->{$id};
 	}
     }
@@ -484,90 +515,95 @@ sub _initialize_delete
     }
 }
 
-
 ##
-# $self->_initialize_up;
+# $self->_initialize_move;
 # ----------------------
 #   Scans the CGI parameters to check if a component has
-#   to be moved up
+#   to be moved
 ##
-sub _initialize_up
-{
-    my $self = shift;
-    my $cgi  = shift;
-   
-    # Then we need to re-scan the cgi parameters to check if a component
-    # has to move up.
-    foreach my $cgi_param_name ($cgi->param)
-    {
-	my $qBLOCK = quotemeta (BLOCK) . "_";
-	
-	if ($cgi_param_name =~ /^$qBLOCK([0-9]+)_([a-zA-Z]+)_up$/)
-	{
-	    my ($id, $type) = $cgi_param_name =~ /^$qBLOCK([0-9]+)_([a-zA-Z]+)_up$/;
-	    if ($id > 1)
-	    {
-		my $id_1 = $id;
-		my $id_2 = $id - 1,
-		
-		my $component_1 = $self->{block}->{$id_1};
-		my $component_2 = $self->{block}->{$id_2};
-		
-		my $type_1 = _PAM_()->{ref $component_1};
-		my $type_2 = _PAM_()->{ref $component_2};
+sub _initialize_move {
+    my ($self, $cgi) = @_;
 
-		$self->{block}->{$id_1} = $component_2;
-		$self->{block}->{$id_2} = $component_1;
+    # look for an indicator that a move button was clicked
+    foreach my $name ($cgi->param()) {
+	next unless $name =~ /^$QBLOCK([0-9]+)_([a-zA-Z]+)_move_button$/;
+        my ($id, $type) = ($1, $2);
+        
+        # collect target for this move
+        my $target = $cgi->param(BLOCK . "_" . $id . "_" . $type . "_move_target");
+
+        # move the component up or down as requested
+        if ($target =~ /^\+(\d+)$/) {
+            my $distance = $1;
+            $id = $self->_move_component_down($id)
+              for (1 .. $distance);
+
+        } elsif ($target =~ /^-(\d+)$/) {
+            my $distance = $1;
+            $id = $self->_move_component_up($id)
+              for (1 .. $distance);
+
+        } elsif ($target eq 'top') {
+            $id = $self->_move_component_up($id)
+              while ($id != 1);
+
+        } elsif ($target eq 'bottom') {
+            my $last = scalar keys %{$self->{block}};
+            $id = $self->_move_component_down($id)
+              while ($id != $last);
+
+        } else {
+            die "Unable to parse move target: $target.";
+        }
 
-		$component_1->{param_name} = BLOCK . "_" . $id_2 . "_" . $type_1;
-		$component_2->{param_name} = BLOCK . "_" . $id_1 . "_" . $type_2;
-	    }
-	}
     }
 }
 
+# move a component down one
+sub _move_component_down {
+    my ($self, $id) = @_;
+    return $id unless $id < scalar keys %{$self->{block}};
 
-##
-# $self->_initialize_up;
-# ----------------------
-#   Scans the CGI parameters to check if a component has
-#   to be moved down
-##
-sub _initialize_down
-{
-    my $self = shift;
-    my $cgi  = shift;
+    my $id_1 = $id;
+    my $id_2 = $id + 1;
     
-    # Then we need to re-scan the cgi parameters to check if a component
-    # has to move down
-    foreach my $cgi_param_name ($cgi->param)
-    {
-	my $qBLOCK = quotemeta (BLOCK) . "_";
-	
-	if ($cgi_param_name =~ /^$qBLOCK([0-9]+)_([a-zA-Z]+)_down$/)
-	{
-	    my ($id, $type) = $cgi_param_name =~ /^$qBLOCK([0-9]+)_([a-zA-Z]+)_down$/;
-	    if ($id < scalar keys %{$self->{block}})
-	    {
-		my $id_1 = $id;
-		my $id_2 = $id + 1,
-		
-		my $component_1 = $self->{block}->{$id_1};
-		my $component_2 = $self->{block}->{$id_2};
-		
-		my $type_1 = _PAM_()->{ref $component_1};
-		my $type_2 = _PAM_()->{ref $component_2};
-		
-		$self->{block}->{$id_1} = $component_2;
-		$self->{block}->{$id_2} = $component_1;
-		
-		$component_1->{param_name} = BLOCK . "_" . $id_2 . "_" . $type_1;
-		$component_2->{param_name} = BLOCK . "_" . $id_1 . "_" . $type_2;
-	    }
-	}
-    }
+    my $component_1 = $self->{block}->{$id_1};
+    my $component_2 = $self->{block}->{$id_2};
+    
+    my $type_1 = _PAM_()->{ref $component_1};
+    my $type_2 = _PAM_()->{ref $component_2};
+    
+    $self->{block}->{$id_1} = $component_2;
+    $self->{block}->{$id_2} = $component_1;
+    
+    $component_1->{param_name} = BLOCK . "_" . $id_2 . "_" . $type_1;
+    $component_2->{param_name} = BLOCK . "_" . $id_1 . "_" . $type_2;
+    
+    return $id_2;
 }
 
+# move a component up one
+sub _move_component_up {
+    my ($self, $id) = @_;
+    return $id unless $id > 1;
+
+    my $id_1 = $id;
+    my $id_2 = $id - 1;
+    
+    my $component_1 = $self->{block}->{$id_1};
+    my $component_2 = $self->{block}->{$id_2};
+    
+    my $type_1 = _PAM_()->{ref $component_1};
+    my $type_2 = _PAM_()->{ref $component_2};
+    
+    $self->{block}->{$id_1} = $component_2;
+    $self->{block}->{$id_2} = $component_1;
+    
+    $component_1->{param_name} = BLOCK . "_" . $id_2 . "_" . $type_1;
+    $component_2->{param_name} = BLOCK . "_" . $id_1 . "_" . $type_2;
+    
+    return $id_2;
+}
 
 ##
 # $self->_initialize_add;
@@ -585,8 +621,7 @@ sub _initialize_add
     if (defined $cgi->param ('_new_component_add'))
     {
 	my $cgi_param_name = $cgi->param ('_new_component');
-	my $qBLOCK = quotemeta (BLOCK) . '_';
-	my ($id, $type) = $cgi_param_name =~ /^$qBLOCK([0-9]+)_([a-zA-Z]+)$/;
+	my ($id, $type) = $cgi_param_name =~ /^$QBLOCK([0-9]+)_([a-zA-Z]+)$/;
 	my $nb = $cgi->param ('_new_component_amount');
 	my $map = _MAP_();
 	my $class = $map->{$type};
@@ -627,6 +662,7 @@ sub add_component
     $component->{parent_id}    = $parent->id();
     $component->{position}     = $next_id;
     $self->{block}->{$next_id} = $component;
+    $self->_initialize_uri_names();
 }
 
 
@@ -662,6 +698,25 @@ sub component
     return $self->{block}->{$id};
 }
 
+# returns the current count of live components
+sub component_count {
+    my $self = shift;
+    return scalar keys %{$self->{block}};
+}
+
+# working replacement for $component->position which is unreliable for
+# some reason
+sub component_position {
+    my $self      = shift;
+    my $component = shift;
+    my $block     = $self->{block};
+    my @keys = sort { $a <=> $b } keys %$block;
+    for (my $i = 0; $i < @keys; $i++) {
+        return $i 
+          if $component->{param_name} eq $block->{$keys[$i]}->{param_name};
+    }
+    return undef;
+}
 
 ##
 # $self->edit;
@@ -684,7 +739,7 @@ sub edit
     foreach (sort keys %{$pam})
     {
 	my $type = $pam->{$_};
-	push @add, { label => $type, name => BLOCK . "_" . $next_id . "_" . $type };
+	push @add, { label => $type, name => BLOCK . "_" . $next_id . "_" . $type, type => {$type => 1}};
     }
     
     # build the add component box
