@@ -82,7 +82,7 @@ sub showable_class
 ##
 # $self->is_showable()
 # --------------------
-# Returns TRUE if this object is hidden, FALSE otherwise.
+# Returns FALSE if this object is hidden or a versioned backup, TRUE otherwise.
 ##
 sub is_showable
 {
@@ -92,6 +92,21 @@ sub is_showable
 		 $self->is_deleted_version() );
 }
 
+=pod
+
+  $self->is_showable_to_editor()
+
+Returns FALSE if this object is a versioned backup, TRUE otherwise.
+
+=cut
+
+sub is_showable_to_editor
+{
+    my $self = shift;
+    return not ( $self->is_modified_version() ||
+		 $self->is_deleted_version() );
+}
+
 
 ##
 # $self->is_hidden();
@@ -1092,6 +1107,21 @@ sub ancestors
 }
 
 
+##
+# $self->root_child();
+# --------------------
+# Returns the ancestor of this document that is a child of the root document.
+##
+sub root_child 
+{
+    my $self = shift;
+    my $ancestors = $self->ancestors;
+    return unless $ancestors;
+    return $self unless $ancestors->[1];
+    return $ancestors->[1];
+}
+
+
 ##
 # $self->position();
 # ------------------
@@ -1676,11 +1706,35 @@ sub children
 sub children_showable
 {
     my $self = shift;
-    my @res  = map { $_->is_showable() ? $_ : () } $self->children;
+    my @res;
+    if (flo::Standard::current_user && flo::Standard::current_user->is_editor)
+    {
+        @res  = map { $_->is_showable_to_editor() ? $_ : () } $self->children;
+    }
+    else
+    {
+        @res  = map { $_->is_showable() ? $_ : () } $self->children;
+    }
     return wantarray ? @res : \@res;
 }
 
 
+##
+# $self->children_showable_percent_each()
+# ---------------------------------------
+# returns 25% if there are 4 children, 12.5% if there are eight etc..
+# css percentages are decimal numbers, not necessarily integers
+# http://www.w3.org/TR/REC-CSS2/syndata.html#value-def-number
+##
+sub children_showable_percent_each
+{
+    my $self = shift;
+    my $children_showable = $self->children_showable;
+    my $number = @$children_showable || return;
+    return (100 / $number) . '%';
+}
+
+
 ##
 # $self->cousins_homonyms();
 # --------------------------
