@@ -45,7 +45,7 @@
 #       my $prefs = new flo::User::Preferences( User => $user );
 #
 #       my @langs = $prefs->language_names;
-#       $prefs->set_language_preference($langs[0], LIKES);
+#       $prefs->set_language_preference($langs[0], LIKE);
 #
 #       my @gens = $prefs->general_names;
 #       my $val  = $prefs->general_preference($gens[4]);
@@ -89,7 +89,7 @@ use base qw/ Exporter /;
 ##
 
 ## Version (from CVS revision string).
-our $VERSION = (split(/ /, '$Revision: 1.11.2.9 $'))[1];
+our $VERSION = (split(/ /, '$Revision: 1.11.2.14 $'))[1];
 
 
 ## Boolean switch controlling debug printing to STDERR.
@@ -445,6 +445,19 @@ sub general_name_exists($)
 }
 
 
+##
+# $obj->set_default_language;
+# ---------------------------
+#   If there is only one language in use, sets language preference to that,
+#   otherwise does nothing.
+##
+sub set_default_language
+{
+    my $self = shift;
+    my @langs = $self->language_names;
+    $self->set_language_preference ($langs[0], LIKE) if (scalar @langs == 1);
+}
+
 
 ##
 # $val = $obj->language_preference($name);
@@ -664,7 +677,7 @@ sub audience_by_id($$)
 # @auds = $obj->audiences;
 # ------------------------
 #   Returns a list of all flo::Record::Audience objects available
-#   to be set, ordered by name, or undef on error.
+#   to be set, ordered by label, or undef on error.
 ##
 sub audiences($)
 {
@@ -674,13 +687,61 @@ sub audiences($)
     unless (exists $self->{_audiences_})
     {
 	my @auds   = load flo::Record::Audience( All => 1 );
-	@AUDIENCES = sort { $a->name cmp $b->name } @auds;
+	@AUDIENCES = sort { $a->label cmp $b->label } @auds;
 	$self->{_audiences_} = [@AUDIENCES];
     }
 
     return wantarray ? @{$self->{_audiences_}} : $self->{_audiences_};
 }
 
+##
+# @audiences = $obj->audience_categories();
+# ------------------------------------
+#   Returns a sorted list of all audience categories.
+#   Categories are implicit when a 'label' has ':' in it.
+#   Otherwise items will be in category ''.
+##
+sub audience_categories
+{
+    my $self = shift;
+
+    my $categories;
+    for my $audience (@{$self->audiences})
+    {
+        $categories->{$audience->category} = undef;
+    }
+
+    my @audience_categories;
+    for (keys %{$categories}) { push @audience_categories, $_ };
+
+    @audience_categories = sort { $a cmp $b } @audience_categories;
+
+    return wantarray ? @audience_categories : [@audience_categories];
+}
+
+##
+# @auds = $obj->audiences_by_category ('Fruit');
+# ------------------------
+#   Returns a list of all flo::Record::Audience objects available
+#   to be set that match the specified category, ordered by label, or undef on error.
+##
+sub audiences_by_category
+{
+    my $self = shift;
+    my $category = shift;
+
+    my @AUDIENCES;
+    my @AUDIENCE_BY_CATEGORY;
+
+    my @auds   = $self->audiences;
+    @AUDIENCES = sort { $a->label cmp $b->label } @auds;
+    for (@AUDIENCES)
+    {
+        push @AUDIENCE_BY_CATEGORY, $_ if $_->category eq $category;
+    }
+
+    return wantarray ? @AUDIENCE_BY_CATEGORY : [@AUDIENCE_BY_CATEGORY];
+}
 
 ##
 # @audiences = $obj->audience_names();
@@ -861,7 +922,12 @@ sub _load_audience_prefs($)
 {
     my $self  = shift;
     my @prefs = load flo::Record::Preference::Audience(User => $self->user);
-    my %hash  = map { $self->audience_by_id($_->audience_id)->name => $_ } @prefs;
+    my @ok_prefs = ();
+    for (@prefs)
+    {
+        push (@ok_prefs, $_) if ($self->audience_by_id($_->audience_id));
+    }
+    my %hash  = map { $self->audience_by_id($_->audience_id)->name => $_ } @ok_prefs;
     $self->_set_audience_prefs(\%hash);
 }
 
