@@ -33,6 +33,7 @@ use utf8;
 use base qw /flo::Component
 	     flo::editor::Mixin::compute_name
 	     flo::editor::Mixin::normalize_name
+	     flo::editor::Mixin::hr_name
 	     flo::editor::Mixin::upload_file/;
 
 
@@ -78,7 +79,8 @@ sub preferred_extension
     my $self  = shift;
     my $file  = $self->{file} || return '';
     $file =~ s/^.*\///;
-    my ($ext) = $file =~ /.*?\.(.*)/;
+    my ($ext) = $file =~ /.*\.(.*)/;
+    $ext = $1 if ($file =~ /\.([^.]+\.gz)$/);
     $ext ||= $self->type();
     return $ext;
 }
@@ -142,8 +144,7 @@ sub new
                 
 		my $self_title = $self->{title};
 		$self_title ||= do {
-		    my $res = $new_file_friendly;
-		    $res =~ s/\..*//;
+                    my $res = $class->hr_name ($self->normalize_name_nodot ($new_file_no_slash));
 		    $res;
 		};
 	
