@@ -23,6 +23,7 @@
 # -------------------------------------------------------------------------------------
 package flo::editor::Headlines;
 use flo::Record::Editor;
+use flo::Record::Audience;
 use Text::Unidecode;
 use MKDoc::Config;
 use flo::Editor;
@@ -30,6 +31,7 @@ use flo::Standard;
 use MKDoc::CGI;
 use MKDoc::Util::Text2HTML;
 use flo::RedirectManager;
+use MKDoc::Util::LinkParser;
 use strict;
 use utf8;
 
@@ -41,6 +43,8 @@ use base qw /flo::Component
 	     flo::editor::Mixin::compute_name
 	     flo::editor::Mixin::normalize_name/;
 
+# keep a global link parser around for use with from_path
+my $link_parser = MKDoc::Util::LinkParser->new();
 
 sub preferred_extension { 'html' };
 
@@ -48,12 +52,82 @@ sub preferred_extension { 'html' };
 sub _initialize
 {
     my $self = shift;
-    my $args = $self->cgi_args();
+    my $args = $self->cgi_args_multi();
+
     $self->{title}         = $args->{title}           || '';
-    $self->{from_path}     = $args->{'from_path'}     || '';
     $self->{max_headlines} = $args->{'max_headlines'} || '';
     $self->{leaf_only}     = $args->{'leaf_only'}     || '';
     $self->{mode}          = $args->{'mode'}          || 'newest';
+    $self->{audience_on}   = $args->{audience_on}     || 0;
+
+    if ($args->{from_path}) {
+        $self->parse_path($args->{from_path});
+    } else {
+        $self->{from_path} = '';
+    }
+
+    # set audiences, wrapping a single value in an array ref
+    $self->{audiences} = [];
+    $self->{audiences} = 
+      ref $args->{audiences} ? $args->{audiences} : [ $args->{audiences} ]
+        if $args->{audiences};
+}
+
+# lookup path using the link-parser, resulting in a document ID
+sub parse_path {
+    my ($self, $url) = @_;
+    $link_parser->parse($url);
+
+    if ($link_parser->is_internal and $link_parser->is_valid) {
+        # store just the document_id, the rest isn't useful
+        $self->{from_document_id} = $link_parser->document_id;
+
+        # store the canonical rendered form in from_path
+        $self->{from_path} = $self->render_from_path;
+    } else {
+        # store the bad path, validate_path will pick it up on save
+        $self->{from_path} = $url;
+    }
+}
+
+# template rendering for the path
+sub render_from_path {
+    my $self = shift;
+    if ($self->{from_document_id}) {
+        # lookup the path for this document
+        my $dbh    = lib::sql::DBH->get;
+        my ($path) = $dbh->selectrow_array('SELECT Full_Path 
+                                            FROM Document WHERE ID = ?',
+                                           undef, $self->{from_document_id});
+        return $path;
+    } else {
+        return $self->{from_path} || '';
+    }
+}
+
+
+# returns an array of audience objects (flo::Record::Audience)
+sub audiences { [ flo::Record::Audience->load( All => 1 ) ] }
+
+# returns true if audience matching is on
+sub audience_on { 
+    my $self = shift;
+    return 1 if $self->{audience_on};
+    return undef;
+}
+
+# returns true if audience matching is off
+sub audience_off { 
+    my $self = shift;
+    return 1 unless $self->{audience_on};
+    return undef;
+}
+
+# returns true if a particular audience is selected
+sub audience_selected { 
+    my ($self, $audience) = @_;
+    return 1 if grep { $_ == $audience->id } @{$self->{audiences}};
+    return undef;
 }
 
 # getter for mode
@@ -96,7 +170,19 @@ sub validate
     return $self->validate_title() &
            $self->validate_from_path() &
 	   $self->validate_max_headlines() &
-           $self->validate_mode;
+           $self->validate_mode &
+           $self->validate_audiences;
+}
+
+# check audiences values
+sub validate_audiences {
+    my $self = shift;
+
+    if ($self->{audience_on} and not @{$self->{audiences}}) {
+	new MKDoc::Ouch 'component/headlines/audiences_missing';
+	return 0;
+    }
+    return 1;
 }
 
 # check mode value
@@ -122,8 +208,19 @@ sub validate_title
 }
 
 
-sub validate_from_path
-{
+sub validate_from_path {
+    my $self = shift;
+    unless ($self->{from_path}) {
+	new MKDoc::Ouch 'component/headlines/from_path_empty';
+	return 0;
+    }
+
+    $link_parser->parse($self->{from_path});
+    unless ($link_parser->is_internal and $link_parser->is_valid) {
+        new MKDoc::Ouch 'component/headlines/from_path_invalid';
+        return 0;
+    }
+
     return 1;
 }
 
@@ -359,6 +456,18 @@ sub _default_headlines_upcoming {
     if ($self->leaf_only()) {
         @res = map { my @children = $_->{document}->children_showable(); @children ? () : $_ } @res;
     }
+
+    # limit to particular set of audiences if audience matching is on
+    if ($self->{audience_on} and $self->{audiences}) {
+        my %ok = map { ($_, 1) } @{$self->{audiences}};
+        my @passed;
+        foreach my $row (@res) {
+            my $doc = $row->{document};
+            push @passed, $row
+              if grep { $ok{$_->id} } $doc->audiences;
+        }
+        @res = @passed;
+    }
  
     # limit to max number of headlines
     @res = splice @res, 0, $self->max_default_headlines();
@@ -394,6 +503,18 @@ sub _default_headlines_newest {
     {
        @res = map { my @children = $_->children_showable(); @children ? () : $_ } @res;
     }
+
+    # limit to particular set of audiences if audience matching is on
+    if ($self->{audience_on} and $self->{audiences}) {
+        my %ok = map { ($_, 1) } @{$self->{audiences}};
+        my @passed;
+        foreach my $doc (@res) {
+            push @passed, $doc 
+              if grep { $ok{$_->id} } $doc->audiences;
+        }
+        @res = @passed;
+    }
+        
  
     # limit to max number of headlines
     @res = splice @res, 0, $self->max_default_headlines();
@@ -427,6 +548,9 @@ sub personalized_headlines
     
     my $mode = $self->{mode};
 
+    # no personalized headlines if audience matching is enabled
+    return [] if $self->{audience_on};
+
     # concoct SQL needed for upcoming or newest mode
     my ($extra_from, $extra_where, $extra_select, $order_by, $group_by);
     if ($mode eq 'upcoming') {
