@@ -93,7 +93,8 @@ sub preferred_extension
     my $self  = shift;
     my $file  = $self->{image} || return 'none';
     $file =~ s/^.*\///;
-    my ($ext) = $file =~ /.*?\.(.*)/;
+    my ($ext) = $file =~ /.*\.(.*)/;
+    $ext = $1 if ($file =~ /\.([^.]+\.gz)$/);
     $ext ||= $self->type();
     return $ext;
 }
@@ -111,18 +112,12 @@ sub new
     # if CGI is defined, then we probably want to do some stuff
     if (defined $cgi)
     {
-	# update alt field
-	if (defined $cgi->param ($param_name . "_alt"))
-	{
-	    my $alt = $cgi->param ($param_name . "_alt");
-	    $self->{title} = $alt;
-	}
-	else
-	{
-	    my $empty = "";
-	    $self->{title} = $empty;
-	}
-	
+	# update fields
+        $self->{title}       = $cgi->param($param_name . "_alt");
+        $self->{title}       = "" unless defined $self->{title};
+        $self->{description} = $cgi->param($param_name . "_description");
+        $self->{description} = "" unless defined $self->{description};
+
 	# if the upload field is not empty, then the user
 	# just uploaded a new image
 	if (defined $cgi->param ($param_name . "_upload"))
@@ -148,7 +143,7 @@ sub new
 		$self->{image} = "$dir/$new_image_friendly";
 		$self->{title} =
 		    $cgi->param ($param_name . "_alt")    ||
-		    $class->hr_name ($new_image_friendly) ||
+		    $class->hr_name ($self->normalize_name_nodot ($new_image_no_slash)) ||
 		    'image_title';
 	    }
 	}
@@ -199,7 +194,7 @@ sub URI
 sub description
 {
     my $self = shift;
-    return $self->{title};
+    return $self->{description};
 }
 
 
