@@ -26,12 +26,18 @@ use flo::Editor;
 use flo::Standard;
 use MKDoc::CGI;
 use MKDoc::Util::Text2HTML;
+use MKDoc::Util::LinkParser;
 use strict;
 
 use base qw /flo::Component
 	     flo::editor::Mixin::compute_name
 	     flo::editor::Mixin::normalize_name/;
 
+# keep a global link parser around
+my $link_parser = MKDoc::Util::LinkParser->new();
+
+# placeholder for the URL entry field
+my $EMPTY_URL = 'http://';
 
 sub preferred_extension { 'link' }
 
@@ -47,12 +53,50 @@ sub _initialize
     my $self = shift;
     my $args = $self->cgi_args()      || return;
 
-    $self->{url}         = $args->{'url'}         || 'http://';
     $self->{title}       = $args->{'title'}       || '';
     $self->{description} = $args->{'description'} || '';
+
+    if ($args->{url}) {
+        $self->parse_url($args->{url});
+    } else {
+        $self->{url} = $EMPTY_URL;
+    }
 }
 
 
+sub parse_url {
+    my ($self, $url) = @_;
+    $link_parser->parse($url);
+
+    if ($link_parser->is_internal and $link_parser->is_valid) {
+        # store relevent details for an internal link (invalid
+        # links are handled by validate_url)
+        $self->{internal_link} = $link_parser->freeze();
+
+        # reflect the canonical form in the UI
+        $self->{url}           = $link_parser->as_string;
+    } else {
+        # store just the url as entered for external links
+        delete $self->{internal_link};
+        $self->{url} = $url;
+    }
+}
+
+# template rendering for the URL
+sub render_url {
+    my $self = shift;
+    if ($self->{internal_link}) {
+        $link_parser->thaw($self->{internal_link});
+        return $link_parser->as_string;
+    } else {
+        return $self->{url};
+    }
+}
+
+sub is_internal {
+    return shift->{internal_link} ? 1 : 0;
+}
+
 sub validate
 {
     my $self = shift;
@@ -70,10 +114,16 @@ sub validate
 sub validate_url
 {
     my $self = shift;
-    $self->{url} and $self->{url} ne 'http://' or do {
+    $self->{url} and $self->{url} ne $EMPTY_URL or do {
 	new MKDoc::Ouch 'component/link/url_empty';
 	return 0;
     };
+
+    $link_parser->parse($self->{url});
+    if ($link_parser->is_internal and not $link_parser->is_valid) {
+        new MKDoc::Ouch 'component/link/invalid_internal_url';
+        return 0;
+    }
     
     return 1;
 }
