@@ -86,7 +86,8 @@ sub preferred_extension
     my $self  = shift;
     my $file  = $self->{image} || return 'jpg';
     $file =~ s/^.*\///;
-    my ($ext) = $file =~ /.*?\.(.*)/;
+    my ($ext) = $file =~ /.*\.(.*)/;
+    $ext = $1 if ($file =~ /\.([^.]+\.gz)$/);
     $ext ||= $self->type();
     return $ext;
 }
@@ -233,6 +234,7 @@ sub new
 		$new_image_no_slash =~ s/^.*\\//g;
 		
 		my $new_image_friendly  = $self->normalize_name ($new_image_no_slash);
+                return $self unless $new_image_friendly =~ /\.(jpeg|jpg|jfif|jpe)$/i;
 		
 		my $dir           = $self->compute_name;
 		my $dir_thumbnail = "$dir/thumbnail";
@@ -258,11 +260,14 @@ sub new
 		    my $scaled_max_y = $config->get ('scaled_max_y');
 		    my $thumb_max_x  = $config->get ('thumb_max_x');
 		    my $thumb_max_y  = $config->get ('thumb_max_y');
+                    my $quality      = $config->get ('quality');
 		    
 		    my $image = new Image::Magick;
 		    my $retcode = $image->Read ("$full_dir/$new_image_friendly");
 		    die $retcode if (defined $retcode and $retcode =~ /exception/i);
 		    
+                    $image->Strip; # requires $Image::Magick::VERSION >= 5.5.8
+                    $image->Set (quality => $quality);
 		    $image->Resize (geometry => $scaled_max_x . 'x' . $scaled_max_y);
 		    $image->Write ("$full_dir_scaled/$new_image_friendly");
 		    $image->Resize (geometry => $thumb_max_x  . 'x' . $thumb_max_y);
@@ -278,7 +283,7 @@ sub new
 
 		$self->{alt} ||=
 		    $cgi->param ($param_name . "_alt")    ||
-		    $class->hr_name ($new_image_friendly) ||
+		    $class->hr_name ($self->normalize_name_nodot ($new_image_no_slash)) ||
 		    'image_title';
 	    }
 	}
