@@ -25,7 +25,7 @@ package flo::editor::Text;
 use flo::Editor;
 use flo::Standard;
 use MKDoc::CGI;
-use MKDoc::XML::Tagger;
+use MKDoc::XML::Tagger::Preserve;
 use MKDoc::Text::Structured;
 use strict;
 
@@ -62,6 +62,7 @@ sub new
 	$self->{data} = "";
     }
     
+    $self->_initialize;
     return $self;
 }
 
@@ -73,19 +74,8 @@ sub html
     $data = MKDoc::Text::Structured::process ($data);
     $data = "<div class=\"text\">$data</div>";
     
-    my @link = flo::Editor->links();
-    my @newl = ();
-    for (@link) {
-        my $new = { %{$_} };
-        $new->{_expr} = delete $new->{expr};
-        $new->{_tag}  = 'a';
-        $new->{title} = delete $new->{desc};
-        push @newl, $new;
-    }
-
-    my $res = MKDoc::XML::Tagger->process_data ($data, @newl);
-    # $res =~ s/^<div>//;
-    # $res =~ s/<\/div>$//;
+    my @newl = $self->links_for_tagger();
+    my $res = MKDoc::XML::Tagger::Preserve->process_data (['a'], $data, @newl);
     return $res;
 }
 
