@@ -63,6 +63,22 @@ sub audiences
 }
 
 
+##
+# $class->templates;
+# ------------------
+# Returns a structure which can be used within a template to choose
+# a document template from a select box.
+#
+# See MKDoc::Util::TemplateFinder::list_structure()
+##
+sub templates
+{
+    my $self    = shift;
+    my $default = $self->form_input ('template') || $self->parent->popular_children_template || shift || 'default';
+    return MKDoc::Util::TemplateFinder::list_structure ('document', $default);
+}
+
+
 ##
 # $self->http_post;
 # -----------------
@@ -84,7 +100,7 @@ sub http_post
 	Lang          => $cgix->param ('lang'),
 	Rights        => $cgix->param ('rights'),
 	Publisher     => $cgix->param ('publisher'),
-	Template      => $current_document->popular_children_template(),
+	Template      => $cgix->param ('template'),
 	Cache_Control => $cgix->param ('cache_control'),
        );
     
@@ -103,6 +119,7 @@ sub http_post
     flo::RedirectManager->document_added ($document->path());
     
     $self->_set_audiences_from_cgi ($document);
+    $self->_set_groups_from_cgi ($document);
     
     # add content if we passed any
     my $editor = _new flo::Editor (cgi => $cgix);
@@ -160,7 +177,7 @@ sub form_input
     my $self   = shift;
     my $param  = shift;
     my $object = shift || current_document();
-    if ($param =~ /^(title|name|keywords|description)$/)
+    if ($param =~ /^(title|name|keywords|description|template)$/)
     {
 	my $cgi = cgi;
 	return $cgi->param ($param);
