@@ -230,6 +230,70 @@ sub unselected_langs
     return (wantarray) ? @unselected : \@unselected;
 }
 
+##
+# $self->groups;
+# -----------------
+#   Returns all groups related to the current document,
+#   but adds a 'checked' flag for those which are checked and a 
+#   'readonly' flag for those that should not be editable.
+##
+sub groups {
+    my $self = shift;
+    my $dbh = lib::sql::DBH->get();
+    my $results = $dbh->selectall_arrayref(
+      'SELECT ID, Name, Description FROM Grp ORDER BY name');
+
+    my @groups;
+    foreach my $row (@$results) {
+        my %group = (id          => $row->[0],
+                     name        => $row->[1],
+                     description => $row->[2]);
+        $self->_group_selection(\%group);
+        push(@groups, \%group);
+    }
+
+    return wantarray ? @groups : \@groups;
+}
+
+# determine whether a group is selected, marking it readonly if the
+# selection comes from a parent.  Also mark readonly if the editor
+# isn't a member of the group.
+sub _group_selection {
+    my ($self, $group) = @_;
+    my $dbh            = lib::sql::DBH->get();
+    my $document       = flo::Standard::current_document();
+    my $group_id       = $group->{id};
+
+    # get list of all documents to check
+    my @documents = ($document, $document->ancestors);
+    
+    # get results for each document for this group
+    my $this_id    = $document->id;
+    foreach my $doc (@documents) {        
+        my ($exists) = $dbh->selectrow_array(
+          'SELECT 1 FROM Document_Grp WHERE Document_ID = ? AND Grp_ID = ?',
+          undef, $doc->id, $group_id);
+        
+        # got a winner, set it checked and reasonly if this came from
+        # up the line
+        if ($exists) {
+            $group->{checked}  = 1;
+            $group->{readonly} = 1 unless $doc->id == $this_id;
+            last;
+        }
+    }
+
+    # check for group membership if the group isn't already readonly
+    unless ($group->{readonly}) {
+        my ($member) = $dbh->selectrow_array(
+          'SELECT 1 FROM Editor_Grp WHERE Editor_ID = ? AND Grp_ID = ?',
+          undef, $self->user->id, $group_id);
+        $group->{readonly} = 1 unless $member;
+    }
+}
+
+    
+
 
 ##
 # $self->audiences;
@@ -251,6 +315,57 @@ sub audiences
 }
 
 
+##
+# $self->audience_categories();
+# ------------------------------------
+#   Returns a sorted list of all audience categories.
+#   Categories are implicit when a 'label' has ':' in it.
+#   Otherwise items will be in category ''.
+##
+sub audience_categories
+{
+    my $self = shift;
+
+    my $categories;
+    for my $audience (@{$self->audiences})
+    {
+        $categories->{$audience->category} = undef;
+    }
+
+    my @audience_categories;
+    for (keys %{$categories}) { push @audience_categories, $_ };
+
+    @audience_categories = sort { $a cmp $b } @audience_categories;
+
+    return wantarray ? @audience_categories : [@audience_categories];
+}
+
+
+##
+# $self->audiences_by_category ('Fruit');
+# ------------------------
+#   Returns a list of all flo::Record::Audience objects available
+#   to be set that match the specified category, ordered by label, or undef on error.
+##
+sub audiences_by_category
+{
+    my $self = shift;
+    my $category = shift;
+
+    my @AUDIENCES;
+    my @AUDIENCE_BY_CATEGORY;
+
+    my @auds   = $self->audiences;
+    @AUDIENCES = sort { $a->label cmp $b->label } @auds;
+    for (@AUDIENCES)
+    {
+        push @AUDIENCE_BY_CATEGORY, $_ if $_->category eq $category;
+    }
+
+    return wantarray ? @AUDIENCE_BY_CATEGORY : [@AUDIENCE_BY_CATEGORY];
+}
+
+
 ##
 # $self->activate;
 # ----------------
@@ -324,6 +439,7 @@ sub http_post
     $document->validate() || return $self->http_get();
     $document->save();
     $self->_set_audiences_from_cgi ($document);
+    $self->_set_groups_from_cgi ($document);
     
     # we have finished changing the properties, let's redirect
     # to the current document
@@ -361,6 +477,36 @@ sub _set_audiences_from_cgi
     $document_audience_table->insert ( Document_ID => $document->id(), Audience_ID => $_ ) for (@valid_ids);
 }
 
+##
+# $self->_set_groups_from_cgi;
+# -------------------------------
+#   Sets the groups in the 'Document_Grp' table from the CGI object.
+##
+sub _set_groups_from_cgi {
+    my $self        = shift;
+    my $document    = shift;
+    my $document_id = $document->id;
+    my $cgi         = flo::Standard::cgi->new;
+    my $dbh         = lib::sql::DBH->get();    
+
+    my %checked = map { $_ => 1 } $cgi->param('group');
+    my %groups  = map { $_->{id} => $_ } $self->groups();
+    foreach my $group_id (keys %groups) {
+        # don't touch readonly settings
+        next if $groups{$group_id}->{readonly};
+
+        # clear the current value
+        $dbh->do('DELETE FROM Document_Grp
+                  WHERE Document_ID = ? AND Grp_ID = ?',
+                 undef, $document_id, $group_id);
+
+        # set if checked
+        $dbh->do('INSERT INTO Document_Grp (Document_ID, Grp_ID) VALUES (?,?)',
+                 undef, $document_id, $group_id)
+          if $checked{$group_id};
+    }
+}
+
 
 ##
 # $self->lang;
