@@ -47,6 +47,11 @@ sub http_post
 {
     my $self = shift;
     my $user_edit = $self->user_edit();
+
+    # clear grp assignments
+    my $editor_group_t = flo::Standard::table ('Editor_Grp');
+    $editor_group_t->delete(Editor_ID => $user_edit->id);
+
     $user_edit->delete();
 
     use flo::plugin::Admin::UserList;
