@@ -70,6 +70,7 @@ sub http_post
 
     $user_edit->save();
     $self->insert_base_document ($user_edit->id());
+    $self->insert_groups ($user_edit->id());
     $self->{ok} = 1;
     return $self->http_get();
 }
@@ -88,6 +89,19 @@ sub insert_base_document
     $base_document_t->insert (Editor_ID => $id, Document_ID => $_) for (@base_documents);
 }
 
+# create entries in Editor_Grp for group choices
+sub insert_groups
+{
+    my ($self, $id) = @_;
+    my $cgi         = flo::Standard::cgi();
+    my @groups      = $cgi->param('groups');
+    
+    my $editor_group_t = flo::Standard::table ('Editor_Grp');
+    $editor_group_t->delete(Editor_ID => $id);
+    $editor_group_t->insert(Editor_ID => $id, Grp_ID => $_) 
+      for @groups;
+}
+
 
 sub documents
 {
@@ -102,7 +116,20 @@ sub documents
     return wantarray ? @res : \@res;
 }
 
+# return list of available groups for template
+sub groups {
+    my $self = shift;
+    my $document_t = flo::Standard::table ('Grp');
+    my @res = $document_t->select (
+	cols => ['ID', 'Name' ],
+	sort => [ 'Name' ],
+	desc => 0,
+       )->fetch_all();
 
+    return wantarray ? @res : \@res;
+}
+
+# returns true if the document is selected, false if not
 sub is_selected
 {
     my $self = shift;
@@ -114,5 +141,21 @@ sub is_selected
     return;
 }
 
+# returns true if the group is selected, false if not
+sub is_group_selected {
+    my ($self, $id) = @_;
+
+    # look for a row for this editor and group in the Editor_Grp table
+    my $grp_t = flo::Standard::table ('Editor_Grp');
+    my @res = $grp_t->select (
+	cols  => ['Grp_ID'],
+        where => lib::sql::Condition->new(Grp_ID    => $id,
+                                          Editor_ID => $self->user_edit->id)
+                                  )->fetch_all();
+
+    return 1 if @res;
+    return;
+}
+
 
 1;
