@@ -72,7 +72,7 @@ use GPanel::Config;
     my $document_table      = lib::sql::Table->table('Document');
     my $base_document_table = lib::sql::Table->table('Base_Document');
     
-    my $amount = int (rand (2)) + 3;
+    my $amount = int (rand (2)) + 5;
     my @rand = qw /a i u e o
 		   ka ki ku ke ko
 		   sa shi su se so
@@ -142,7 +142,7 @@ use GPanel::Config;
 
 
 {
-    my $amount = int (rand (2)) + 3;
+    my $amount = int (rand (2)) + 5;
     my @rand = qw /a i u e o
 		   ka ki ku ke ko
 		   sa shi su se so
@@ -177,7 +177,7 @@ $MKDoc::Site::Config::CONFIG = do {
     my $dbname = lc($domain_name);
     $dbname =~ s/\W+/_/g;
     
-    $config->{database_user} = $user_name . $inc;
+    $config->{database_user} = $user_name . $inc . int (rand (10000));
     $config->{database_name} = $dbname;
     $config->{database_host} = 'localhost';
     $config->run();
@@ -329,8 +329,6 @@ my $conf = "$conf_dir/$domain_name.conf";
 #open my $fh, ">$conf" or die "Open $conf: $!";
 open $fh, ">$conf" or die "Open $conf: $!";
 my $env = <<END;
-    PerlRequire MKDoc/Handler/Initialize.pm
-    PerlInitHandler MKDoc::Handler::Initialize
 # those env vars were generated by the Makefile, don't touch them!
     PerlSetEnv  PERL5LIB                /usr/local/mkdoc
     PerlSetEnv  MKDOC_DIR               /usr/local/mkdoc
@@ -339,8 +337,10 @@ my $env = <<END;
     SetEnv      SITE_DIR                $site_dir
     SetEnv      PUBLIC_DOMAIN           http://www.$domain_name/
     SetEnv      USER_DOMAIN             http://users.$domain_name/
-    SetEnv      SERVER_ADMIN            "admin\@$domain_name"
-    ServerAdmin "admin\@$domain_name"
+    SetEnv      SERVER_ADMIN            "tech\@webarch.net"
+    ServerAdmin "tech\@webarch.net"
+
+    SetEnv      TZ                      Europe/London
 
     # where to cache stuff
     SetEnv  MKD__CACHE_DIR  "$site_dir/cache"
@@ -381,31 +381,30 @@ my $env = <<END;
     # SetEnv MKD__TIDY_CONF               /usr/local/mkdoc/custom_tidy_config.conf
 END
 my $alias = <<END;
-    RedirectMatch permanent .+/\.sitemap\.html\$ http://www.$domain_name/.sitemap.html
     Alias /.static/css    $site_dir/resources/skin
     Alias /.static        $site_dir/static
-
-    <Location /.server-status>
-        SetHandler server-status
-    </Location>
-    <Location /.perl-status>
-        require valid-user
-        SetHandler perl-script
-        PerlHandler Apache::Status
-    </Location>
-    RedirectMatch ^/.logs\$ http://www.$domain_name/.logs/
-    <Directory $site_dir/logs/>
-        Options Indexes
-    </Directory>
-    Alias /.logs/ $site_dir/logs/
-
     Alias /               /usr/local/mkdoc/cgi/mkdoc.cgi/
+
+    # Custom error documents
+    <Directory $site_dir/resources/skin>
+      Options Indexes Includes MultiViews
+      AddHandler server-parsed .shtml
+      AddType "text/html; charset=UTF-8" .shtml
+      order allow,deny
+      allow from all
+      AllowOverride none
+    </Directory>
+    ErrorDocument 401 /.static/css/401
+    ErrorDocument 403 /.static/css/403
+    ErrorDocument 404 /.static/css/404
+    ErrorDocument 500 /.static/css/500
 END
 my $handler = <<END;
     PerlRequire MKDoc/Handler/Initialize.pm
     PerlInitHandler MKDoc::Handler::Initialize
 END
 print $fh <<END;
+# Editor: vim:syn=apache
 <VirtualHost *>
     ServerName    $domain_name
     ServerAlias   $domain_name.mkdoc.net
@@ -422,6 +421,10 @@ print $fh <<END;
     #CustomLog $site_dir/logs/perl_access_log combined
 $handler
 $env
+    # Redirect requests for server status and logs to the users interface
+    Redirect /.server-status http://users.$domain_name/.server-status
+    Redirect /.perl-status http://users.$domain_name/.perl-status
+    Redirect /.logs http://users.$domain_name/.logs
 $alias
 </VirtualHost>
 
@@ -433,13 +436,59 @@ $alias
     #CustomLog $site_dir/logs/perl_access_log combined
 $handler
 $env
-$alias
-    <Location />
-        PerlAuthenHandler     MKDoc::Handler::Authenticate
+    # apache icons, needed for the logs
+    Alias /icons/ /var/httpd/icons/
+    <Location /.server-status>
+        SetHandler server-status
+        AuthType Basic
+        AuthName "Please enter your user credentials"
+        AuthUserFile /etc/htpasswd
+        require valid-user
+    </Location>
+    <Location /.perl-status>
+        AuthType Basic
+        AuthName "Please enter your user credentials"
+        AuthUserFile /etc/htpasswd
+        require valid-user
+        SetHandler perl-script
+        PerlHandler Apache::Status
+    </Location>
+    <Directory $site_dir/logs/>
+        Options Indexes
+        PerlAuthenHandler MKDoc::Handler::Authenticate
+        AuthName "Please enter your user credentials"
+        AuthType Basic
+        require user admin
+    </Directory>
+    Alias /.logs/ $site_dir/logs/
+    #<Location />
+    #    PerlAuthenHandler     MKDoc::Handler::Authenticate
+    #    AuthName "Please enter your user credentials"
+    #    AuthType Basic
+    #    require valid-user
+    #</Location>
+    <LocationMatch ^\/\$>
+        PerlAuthenHandler MKDoc::Handler::Authenticate
+        PerlAuthzHandler  MKDoc::Handler::GroupAuthz
         AuthName "Please enter your user credentials"
         AuthType Basic
         require valid-user
-    </Location>
+    </LocationMatch>
+    <LocationMatch ^\/[a-zA-Z0-9]>
+        PerlAuthenHandler MKDoc::Handler::Authenticate
+        PerlAuthzHandler  MKDoc::Handler::GroupAuthz
+        AuthName "Please enter your user credentials"
+        AuthType Basic
+        require valid-user
+    </LocationMatch>
+    <LocationMatch ^\/\.(account|admin|sitemap|print|headlines|meta|search)>
+        PerlAuthenHandler MKDoc::Handler::Authenticate
+        PerlAuthzHandler  MKDoc::Handler::GroupAuthz
+        AuthName "Please enter your user credentials"
+        AuthType Basic
+        require valid-user
+    </LocationMatch>
+$alias
 </VirtualHost>
 
 END
