@@ -7,6 +7,9 @@ my $some_arg    = shift || die "Usage: $0 <site_prefix>/<domain_name>
   Example: $0 /var/www/your-site.net\n";
 $some_arg =~ s/\/$//;
 
+use File::Spec;
+$some_arg = File::Spec->rel2abs ($some_arg);
+
 my ($sites, $base_domain) = $some_arg =~ /(.*)\/(.*)/;
 my $db_domain = $base_domain;
 $db_domain =~ s/[^a-z0-9]/_/gi;
